<?php


/**
 * @file
 *  Administration form for OpenERP Proxy API module
 *
 * Copyright (c) 2010 Pinacono Co., Ltd.
 */

/**
 * settings form
 */
function openerp_settings_form() {
  global $user;

  $proxy = OpenERPProxy::getProxy();

  // gathering information
  if ( ($ver = $proxy->request('db', 'server_version')) === FALSE ) {
    drupal_set_message("Cannot get server version - ". join('<br/>', OpenERPProxy::message()), 'error');
  }

  $dbs = _openerp_get_dbs();

  if ( $proxy->login() === FALSE ) {
    drupal_set_message("Cannot login to server - ". join('<br/>', OpenERPProxy::message()), 'error');
    $models = array();
  }
  else {
    $models = _openerp_get_models();
  }

  $url = variable_get('openerp.url', NULL);
  $db  = variable_get('openerp.database', NULL);
  $uid = variable_get('openerp.user', NULL);
  $pwd = OpenERPProxy::decrypt(variable_get('openerp.password', NULL));

  // load previous workbench query, if available
  $wb_defaults = array(
    'model' =>  NULL,
    'fields' => NULL,
    'filter' => NULL,
    'offset' => 0,
    'limit'  => 10,
  );

  if ( isset($_SESSION['openerp.workbench']) && ! empty($_SESSION['openerp.workbench']['query']) ) {
    foreach ( $_SESSION['openerp.workbench']['query'] as $f => $v ) {
      if ( array_key_exists($f, $wb_defaults) ) {
        $wb_defaults[$f] = $v;
      }
    }
  }

  // set the form
  $form = array(
    'openerp' => array(
      '#type'        => 'fieldset',
      '#title'       => 'Server Configuration',
      '#collapsible' => TRUE,
      '#collapsed'   => ! empty($dbs),
      '#description' => t(
        '<h3>Current Server Information:</h3>'.
        '<p>'.
        '<div>Server version: <em>!version</em></div>'.
        '</p>',
        array(
          '!version'  => $ver,
        )),

      'page_size' => array(
        '#type'  => 'textfield',
        '#title' => t('Default Page Size'),
        '#size'  => 5,
        '#maxlength' => 5,
        '#default_value' => variable_get('openerp.default_page_size', 40),
        '#description'   => t('Specify the maximum row of data per page to be displayed in a list (tree) view, or "0" to use configuration from database.'),
      ),

      'url' => array(
        '#type'  => 'textfield',
        '#title' => t('OpenERP XML-RPC Server URL'),
        '#size'  => 64,
        '#maxlength'     => 256,
        '#default_value' => variable_get('openerp.url', 'http://erp.server.com:8069/xmlrpc'),
        '#description'   => t('Specify the OpenERP XML-RPC Server URL. (e.g., http://erp.server.com:8069/xmlrpc'),
      ),

      'submit' => array(
        '#type'     => 'submit',
        '#value'    => 'Save Server Settings',
        '#validate' => array('openerp_server_validate'),
        '#submit'   => array('openerp_server_submit'),
      ),
    ),

    'user' => array(
      '#type'        => 'fieldset',
      '#title'       => 'Default User Configuration',
      '#collapsible' => TRUE,
      '#collapsed'   => empty($dbs) || (isset($db) && isset($uid) && isset($pwd)),
      '#description' => t('Default user to be used as an active user when the current Drupal user do not have an OpenERP account.'),

      'erp-dbname' => array(
        '#type'  => 'select',
        '#title' => t('OpenERP Database'),
        '#default_value' => variable_get('openerp.database', ''),
        '#options'       => (array) $dbs,
        '#description'   => t('Specify the OpenERP database name.'),
      ),

      'erp-user' => array(
        '#type'  => 'textfield',
        '#title' => t('OpenERP User'),
        '#size'  => 30,
        '#maxlength'     => 256,
        '#default_value' => variable_get('openerp.user', ''),
        '#description'   => t('Specify the OpenERP user name.'),
      ),

      'erp-passwd' => array(
        '#type'  => 'password',
        '#title' => t('OpenERP Password'),
        '#size'  => 15,
        '#description' => t('Specify the password for the OpenERP user. Leave both fields blank to keep the old password.'),
      ),

      'erp-passwd-confirm' => array(
        '#type'  => 'password',
        '#title' => t('Confirm Password'),
        '#size'  => 15,
        '#description' => t('Please re-enter the password to confirm.'),
      ),

      'submit' => array(
        '#type'     => 'submit',
        '#value'    => 'Save User Settings',
        '#validate' => array('openerp_user_validate'),
        '#submit'   => array('openerp_user_submit'),
      ),
    ),

    'workbench' => array(
      '#type'        => 'fieldset',
      '#title'       => t('Database Workbench'),
      '#weight'      => 99, // put it at the end of form
      '#collapsible' => TRUE,
      '#collapsed'   => empty($dbs) || empty($_SESSION['openerp.workbench']),
      '#description' => t('Provide quick access to the OpenERP server data.'),

      'model' => array(
        '#type'  => 'select',
        '#title' => t('Model Name'),
        '#options'       => (array) $models,
        '#default_value' => $wb_defaults['model'],
        '#description'   => t('Select the OpenERP model name to retrieve the data'),
      ),

      'fields' => array(
        '#type'  => 'textfield',
        '#title' => t('Fields'),
        '#size'  => 64,
        '#maxlength' => 256,
        '#default_value' => $wb_defaults['fields'],
        '#description' => t('List of field(s) to retrieve, without quoted and seperated by comma (,) or leave blank to get all fields. '.
                            'Note that "id" is the default field. It will always be returned regardless to specify it or not.'),
      ),

      'filter' => array(
        '#type'  => 'textfield',
        '#title' => t('Filter Options'),
        '#size'  => 64,
        '#maxlength' => 256,
        '#default_value' => $wb_defaults['filter'],
        '#description' => t('Additional filter options in PHP code format, e.g., <em>array(\'field1\',\'=\',\'value1\'), array(\'field2\',\'=\',\'value2\'), ...</em>, etc.'),
      ),

      'offset' => array(
        '#type'  => 'textfield',
        '#title' => t('Offset'),
        '#size'  => 10,
        '#maxlength'     => 8,
        '#default_value' => $wb_defaults['offset'],
        '#description'   => t('Specify the data offset'),
      ),

      'limit' => array(
        '#type'  => 'textfield',
        '#title' => t('Limit'),
        '#size'  => 10,
        '#maxlength'     => 8,
        '#default_value' => $wb_defaults['limit'],
        '#description'   => t('Specify the data limit'),
      ),

      'submit' => array(
        '#type'     => 'submit',
        '#value'    => 'Retrieve Data',
        '#validate' => array('openerp_workbench_validate'),
        '#submit'   => array('openerp_workbench_submit'),
      ),
    ),
  );

  if ( ! empty($_SESSION['openerp.workbench']['result']) ) {
    $form['workbench']['result'] = array(
      '#type'        => 'fieldset',
      '#title'       => 'Database Workbench result',
      '#collapsible' => TRUE,
      '#collapsed'   => FALSE,
      '#description' => t('Result from the Database Workbench Query.'),

      'data' => array(
        '#prefix' => '<div id="openerp-workbench-result-data">',
        '#value'  => $_SESSION['openerp.workbench']['result'],
        '#suffix' => '</div>',
      ),
    );
  }

  $_SESSION['openerp.workbench']['result'] = NULL;
  return $form;
}

/**
 * Validate server configuration
 */
function openerp_server_validate($form, &$form_data) {

  $url = parse_url($form_data['values']['url']);
  if ( $url === FALSE || ! isset($url['scheme']) || ! isset($url['host']) ) {
    form_set_error('url', 'A valid URL to OpenERP XMLRPC service is required.');
  }

  if ( ! is_numeric($form_data['values']['page_size']) || ((int) $form_data['values']['page_size'] != $form_data['values']['page_size']) ) {
    form_set_error('page_size', 'Default page size must be a valid integer.');
  }
  elseif ( ((int) $form_data['values']['page_size']) < 0 ) {
    form_set_error('page_size', 'Default page size must be greater than or equal to 0');
  }
}

/**
 * Validate user configuration
 */
function openerp_user_validate($form, &$form_data) {

  $dbs = _openerp_get_dbs();

  if ( ! array_key_exists($form_data['values']['erp-dbname'], $dbs) ) {
    form_set_error('erp-dbname', 'Database not existing');
  }

  if ( empty($form_data['values']['erp-user']) ) {
    form_set_error('erp-user', 'User login id is required');
  }

  $passwd = OpenERPProxy::decrypt(variable_get('openerp.password', NULL));

  if ( ! empty($form_data['values']['erp-passwd']) ) {
    if ( $form_data['values']['erp-passwd'] != $form_data['values']['erp-passwd-confirm'] ) {
      form_set_error('erp-passwd-confirm', 'Password confirmation is not match the password field');
      return;
    }

    $password = $form_data['values']['erp-passwd'];
  }

  $proxy = OpenERPProxy::getProxy();

  if ( $proxy->request('common', 'login', array($form_data['values']['erp-dbname'], $form_data['values']['erp-user'], $password)) === FALSE ) {
    form_set_error('erp-user', 'OpenERP cannot authenticate user. Your OpenERP user id and/or password may be incorrect.', 'error');
  }
}

/**
 * Validate workbench parameters
 */
function openerp_workbench_validate($form, &$form_data) {
  $models = _openerp_get_models();

  if ( ! array_key_exists($form_data['values']['model'], $models) ) {
    form_set_error('model', "Model {$form_data['values']['model']} is not existing");
  }

  if ( ! is_numeric($form_data['values']['offset']) ) {
    form_set_error('offset', "Offset must be a valid integer");
  }

  if ( ! is_numeric($form_data['values']['limit']) || ($form_data['values']['limit'] < 0) ) {
    form_set_error('limit', "Limit must be an integer and must be larger than 0");
  }
}

/**
 * Save server configurations
 */
function openerp_server_submit($form, &$form_data) {
  variable_set('openerp.url', $form_data['values']['url']);
  variable_set('openerp.default_page_size', $form_data['values']['page_size']);
  drupal_set_message(t('Server configurations successfully saved.'));
}

/**
 * Save OpenERP user configurations
 */
function openerp_user_submit($form, &$form_data) {
  if ( ! empty($form_data['values']['erp-passwd']) ) {
    variable_set('openerp.password', OpenERPProxy::encrypt($form_data['values']['erp-passwd']));
  }
  variable_set('openerp.user',     $form_data['values']['erp-user']);
  variable_set('openerp.database', $form_data['values']['erp-dbname']);
  drupal_set_message(t('OpenERP user configurations successfully saved.'));
}

/**
 * workbench handle
 */
function openerp_workbench_submit($form, &$form_data) {
  $filter = array();
  $fields = array();
  $proxy  = OpenERPProxy::getProxy();

  if ( ! empty($form_data['values']['filter']) ) {
    $filter = eval("return array({$form_data['values']['filter']});");
  }

  if ( ! empty($form_data['values']['fields']) ) {
    $fields = explode(',', $form_data['values']['fields']);
    foreach ( $fields as &$f ) {
      $f = trim($f);
    }
  }

  $_SESSION['openerp.workbench'] = array( 'query' => $form_data['values'] );

  try {
    if ( ($r = $proxy->search($form_data['values']['model'], $filter, (int) $form_data['values']['offset'], (int) $form_data['values']['limit'])) === FALSE ) {
      throw new Exception(join("<br/>", OpenERPProxy::message()));
    }

    if ( ($r = $proxy->read($form_data['values']['model'], $r, $fields)) === FALSE ) {
      throw new Exception(join("<br/>", OpenERPProxy::message()));
    }

    if ( count($r) <= 0 ) {
      throw new Exception("There is no data in the model <em>{$form_data['values']['model']}</em>" .( ! empty($form_data['values']['filter']) ? " that match the filter : {$form_data['values']['filter']}." : '.') );
    }
  } catch ( Exception $e ) {
    drupal_set_message($e->getMessage(), 'error');
    return;
  }

  // dump data in a table
  $msg  = "<h3>{$form_data['values']['model']}</h3>".
          "<table>";

  // header
  $msg .= '<tr>';
  foreach ( $r[0] as $name => $val ) {
    $msg .= "<th>$name</th>";
  }
  $msg .= '</tr>';

  // data
  foreach ( $r as $data ) {
    $msg .= '<tr>';
    foreach ( $data as $name => $val ) {
      $msg .= "<td>";
      $msg .= htmlentities( is_array($val) ? implode(', ', $val) :  $val, ENT_QUOTES );
      $msg .= "</td>";
    }
    $msg .= '</tr>';
  }

  $msg .= '</table>';
  $_SESSION['openerp.workbench']['result'] = $msg;
}
